import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_cover_mode_route.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_flashcard_mode_route.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_write_mode_route.dart';
import 'package:cramr_ui_lib/routes/verb_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:cramr_ui_lib/util/pronoun_decorator.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// The Route that allows you to test your knowledge of the verb that you want
/// to learn
class VerbIndicativeTestRoute extends StatelessWidget {
  static const routeName = '/verbIndicativeTestRoute';

  final SettingsController _settingsController;

  const VerbIndicativeTestRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: _settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
          themeMode: _settingsController.themeMode,
          theme: CustomTheme.lightTheme,
          darkTheme: ThemeData.dark(),
          debugShowCheckedModeBanner: false,
          home: Scaffold(
            appBar: AppBar(
              actions: [
                _repeatIcon(context),
                _homeIcon(context),
                _settingsIcon(context)
              ],
              title: _titleCell(context),
            ),
            body: Container(child: _formBody(context)),
            floatingActionButton: _evaluateButton(context),
          ),
        );
      },
    );
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }

  IconButton _homeIcon(BuildContext context) {
    return IconButton(
        key: const Key('home_button'),
        onPressed: () {
          final VerbTestModel testModel =
              Provider.of<VerbTestModel>(context, listen: false);
          testModel.reset();
          Navigator.restorablePopAndPushNamed(
              context, VerbSelectionRoute.routeName);
        },
        icon: const Icon(Icons.home));
  }

  IconButton _repeatIcon(BuildContext context) {
    return IconButton(
        key: const Key('back_button'),
        onPressed: () {
          final VerbTestModel testModel =
              Provider.of<VerbTestModel>(context, listen: false);
          testModel.reset();
          final StudyModeState studyModeState =
              Provider.of<StudyModeState>(context, listen: false);
          final StudyMode? _studyMode = studyModeState.currentStudyMode;
          if (_studyMode != null) {
            switch (_studyMode) {
              case StudyMode.flashCardMode:
                Navigator.popAndPushNamed(
                    context, VerbIndicativeStudyFlashcardModeRoute.routeName);
                break;
              case StudyMode.coverMode:
                Navigator.popAndPushNamed(
                    context, VerbIndicativeStudyCoverModeRoute.routeName);
                break;
              case StudyMode.writtenMode:
                Navigator.popAndPushNamed(
                    context, VerbIndicativeStudyWriteModeRoute.routeName);
                break;
            }
          } else {
            Navigator.popAndPushNamed(context, '/verbIndicativeStudyRoute');
          }
        },
        icon: const Icon(Icons.refresh));
  }

  Wrap _titleCell(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return Wrap(direction: Axis.horizontal, children: [
      const Text(
        "Vervoeging ",
      ),
      Text(
        testModel.verb.infinitive,
      ),
    ]);
  }

  FloatingActionButton _evaluateButton(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return FloatingActionButton(
      key: Key('evaluate_${testModel.verb.infinitive}'),
      child: const Icon(Icons.arrow_forward),
      onPressed: () {
        if (testModel.isComplete()) {
          Navigator.pushNamed(context, '/verbEvaluateRoute');
        }
      },
    );
  }

  Widget _formBody(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return SafeArea(
      child: Scaffold(
          body: ListView.builder(
              key: Key('scroll_test_${testModel.verb.infinitive}'),
              itemCount: testModel.verb.indicative.length,
              scrollDirection: Axis.vertical,
              controller: ScrollController(),
              physics: const ScrollPhysics(
                parent: AlwaysScrollableScrollPhysics(),
              ),
              itemBuilder: (BuildContext context, int index) {
                return Padding(
                  padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
                  child: Card(
                    child: ListTile(
                      visualDensity: VisualDensity.comfortable,
                      contentPadding: const EdgeInsets.fromLTRB(15, 5, 30, 5),
                      dense: true,
                      leading: _pronounCell(context, index),
                      title: _verbCell(context, index),
                    ),
                    elevation: 5.0,
                  ),
                );
              })),
    );
  }

  SizedBox _pronounCell(BuildContext context, int person) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return SizedBox(
      height: 70,
      width: 85,
      child: Container(
        alignment: Alignment.center,
        child: Text(
          PronounDecorator.decoratedPronoun(testModel.targetLanguageIso,
              testModel.pronouns[person], testModel.verb.indicative[person]),
          style: const TextStyle(
            color: Colors.blue,
            fontSize: 25,
          ),
          textAlign: TextAlign.center,
          overflow: TextOverflow.ellipsis,
        ),
      ),
    );
  }

  Widget _verbCell(BuildContext context, int person) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return SizedBox(
      height: 40,
      width: 150,
      child: Container(
        alignment: Alignment.center,
        child: TextField(
          key: Key('test_${testModel.verb.infinitive}_$person'),
          obscureText: false,
          autofocus: false,
          decoration: const InputDecoration(
            border: OutlineInputBorder(),
          ),
          readOnly: false,
          onSubmitted: (String inputValue) {
            testModel.add(inputValue, person);
          },
          onChanged: (String inputValue) {
            testModel.add(inputValue, person);
          },
        ),
      ),
    );
  }
}
