import 'dart:collection';

import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/repo/verb_repository.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// The Route that allows you to select a verb for evaluation
class VerbSelectionRoute extends StatefulWidget {
  static const routeName = '/verbSelectionRoute';

  final SettingsController _settingsController;

  const VerbSelectionRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  State<VerbSelectionRoute> createState() => VerbSelectionRouteState();
}

class VerbSelectionRouteState extends State<VerbSelectionRoute> {
  final HashMap<String, Verb> verbs = VerbRepository.instance().verbs;

  List<String>? filteredVerbs = [];

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: widget._settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
            debugShowCheckedModeBanner: false,
            themeMode: widget._settingsController.themeMode,
            theme: ThemeData(),
            darkTheme: ThemeData.dark(),
            home: SafeArea(
              child: Scaffold(
                appBar: AppBar(
                  actions: [
                    IconButton(
                      key: const Key("searchVerbButton"),
                      onPressed: () async {
                        String? filteredVerb = await showSearch(
                            context: context,
                            delegate: VerbSearchDelegate(),
                            useRootNavigator: true);
                        setState(() {
                          filteredVerbs!.clear();
                          filteredVerbs!.add(filteredVerb!);
                        });
                      },
                      icon: const Icon(Icons.search),
                    ),
                    _settingsIcon(context),
                  ],
                ),
                body: Center(
                  key: const Key('verbSelectionCenter'),
                  child: ListView(
                    key: const Key('verbListView'),
                    scrollDirection: Axis.vertical,
                    controller: ScrollController(),
                    physics: const ScrollPhysics(
                      parent: AlwaysScrollableScrollPhysics(),
                    ),
                    children: _repoButtons(context),
                  ),
                ),
                // This trailing comma makes auto-formatting nicer for build methods.
              ),
            ),
          );
        });
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }

  List<Widget> _repoButtons(BuildContext context) {
    List<Widget> result = [];

    for (var key in filteredVerbs!) {
      if (verbs.containsKey(key)) {
        Verb? value = verbs[key];
        result.add(SizedBox(
          child: Card(
            key: Key('verbCard_$value'),
            child: _verbButton(context, value!),
            margin: const EdgeInsets.fromLTRB(30, 15, 30, 15),
            elevation: 10.0,
          ),
        ));
      }
    }

    return result;
  }

  ListTile _verbButton(BuildContext context, Verb verb) {
    return ListTile(
      key: Key("vervoeg ${verb.infinitive}"),
      visualDensity: VisualDensity.comfortable,
      contentPadding: const EdgeInsets.fromLTRB(30, 10, 30, 30),
      dense: true,
      isThreeLine: true,
      title: Column(
        key: UniqueKey(),
        children: [
          SizedBox(
            key: UniqueKey(),
            height: 40,
            child: Text(
              verb.infinitive,
              key: UniqueKey(),
              style: CustomTheme.lightTheme.textTheme.headline1,
            ),
          ),
        ],
      ),
      subtitle: Column(
        key: UniqueKey(),
        children: [
          SizedBox(
            key: UniqueKey(),
            height: 40,
            child: Text(
              verb.translation,
              key: UniqueKey(),
              style: CustomTheme.lightTheme.textTheme.bodyText2,
            ),
          ),
          _verbIcon(context, verb),
        ],
      ),
    );
  }

  Widget _verbIcon(BuildContext context, Verb verb) {
    return InkWell(
      key: UniqueKey(),
      child: CircleAvatar(
          key: UniqueKey(),
          child: Icon(Icons.play_arrow,
              key: Key('startStudy_${verb.infinitive}'))),
      onTap: () {
        final VerbTestModel model =
            Provider.of<VerbTestModel>(context, listen: false);
        final CardinalitySelection selection =
            widget._settingsController.cardinalitySelection;
        model.initialise(
            CardinalityFilter.filteredList(selection, verb.indicative),
            CardinalityFilter.filteredList(
                selection, VerbRepository.instance().pronouns),
            verb.isoLanguage,
            verb.infinitive,
            verb.translation);
        model.reset();
        Navigator.restorablePopAndPushNamed(
            context, VerbStudymodeSelectionRoute.routeName);
      },
    );
  }

  @override
  void initState() {
    super.initState();

    verbs.forEach((key, value) {
      filteredVerbs!.add(key);
    });
  }
}

class VerbSearchDelegate extends SearchDelegate<String> {
  List<String> allVerbs = [];
  List<String> suggestedVerbs = [];

  VerbSearchDelegate() {
    var verbs = VerbRepository.instance().verbs;
    for (var element in verbs.keys) {
      allVerbs.add(element);
    }
  }

  @override
  List<Widget>? buildActions(BuildContext context) {
    List<Widget> actions = [];
    return actions;
  }

  @override
  Widget? buildLeading(BuildContext context) {
    return null;
  }

  @override
  Widget buildResults(BuildContext context) {
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(result, key: Key("searchResult_$result")),
            onTap: () {
              close(context, result);
            },
          );
        });
  }

  @override
  Widget buildSuggestions(BuildContext context) {
    suggestedVerbs.clear();
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(
              result,
              key: Key("suggestedResult_$result"),
            ),
            onTap: () {
              close(context, result);
            },
          );
        });
  }
}
